## Weighted Olympic Medal Counts
## Interactive Visualization
## R helper functions
##
## Christopher Adolph
## 4 February 2022   faculty.washington.edu/cadolph

## Scrape 2022 data
scrape2022 <- function() {
    library(rvest)
    webpage <- read_html("https://olympics.com/beijing-2022/olympic-games/en/results/all-sports/medal-standings.htm")
    tbls_ls <- webpage %>% html_nodes("table") %>% html_table(fill = TRUE)
    Country <- tbls_ls[[1]]$NOC
    Gold <- tbls_ls[[1]][,3]
    Silver <- tbls_ls[[1]][,4]
    Bronze <- tbls_ls[[1]][,5]
    Totals <- tbls_ls[[1]][,6]
    Country[Country=="Republic of Korea"] <- "South Korea"
    Country[Country=="People's Republic of China"] <- "China"
    Country[Country=="United States of America"] <- "United States"
    Country[Country=="ROC"] <- "Russia"
    res <- as.data.frame(cbind(Gold, Silver, Bronze, Totals), stringsAsFactors = FALSE)
    res <- cbind(Country, res)
    res$Country <- as.character(res$Country)
    names(res) <- c("Country", "Gold", "Silver", "Bronze", "Totals")
    res
}

## Scrape 2018 data 
scrape2018 <- function() {
    library(rvest)
    webpage <- read_html("https://www.pyeongchang2018.com/en/game-time/results/OWG2018/en/general/medal-standings.htm")
    tbls_ls <- webpage %>% html_nodes("table") %>% html_table(fill = TRUE)
    Country <- tbls_ls[[1]]$NOC
    Gold <- tbls_ls[[1]][,3]
    Silver <- tbls_ls[[1]][,4]
    Bronze <- tbls_ls[[1]][,5]
    Totals <- tbls_ls[[1]][,6]
    Country[Country=="Republic of Korea"] <- "South Korea"
    Country[Country=="OAR"] <- "Russia"
    res <- as.data.frame(cbind(Gold, Silver, Bronze, Totals), stringsAsFactors = FALSE)
    res <- cbind(Country, res)
    res$Country <- as.character(res$Country)
    res
}

## Convert Olympiad numbers to year
olympiadW <- function(year) {
    c(seq(1924, 1936, 4), seq(1948, 1992, 4), seq(1994, 2022,4))[year]
}

olympiadS <- function(year) {
    seq(2000, 2012, 4)[year-26]
}

## Exclude countries from each dataframe based on user input
excludeCountry <- function(x, excluded) {
    le <- length(excluded)
    if (le) {
        lx <- length(x)
        for (j in 1:lx) {
            for (i in 1:le) {
                x[[j]] <- x[[j]][x[[j]]$Country!=excluded[i],]
            }
        }
    }
    x
}

## Return string with name(s) of weighted medal winners (multiple if tie)
weightedMedals <- function(x,medals,weights=c(1,0),unit) {
    weights <- c(weights,x)
    wm <- medals%*%weights
    paste(unit[(1:length(wm))[wm==max(wm)]], collapse="\n")
}

## Return string with name(s) of weighted medal winners (multiple if tie)
## using geometric weights 
weightedMedalsK <- function(x,medals,unit) {
    weights <- c(1,x,x^2)
    wm <- medals%*%weights
    paste(unit[(1:length(wm))[wm==max(wm)]], collapse="\n")
}

## Collect winning countries over weighting schemes
##  if a single year, compute all weighting schemes
##  if multiple years, compute all geometric weighting schemes
getWinners <- function(data, precision=0.05) { # 0.05
    ld <- length(data)

    if (ld>1) {
        ## multiple years: all weights
        allK <- seq(0, 1, by=precision)
        nK <- length(allK)
        res <- as.data.frame(matrix(NA, nrow=nK*ld, ncol=4))
        names(res) <- c("winner","color","year","k")
        for (i in 1:ld) {
            unit <- data[[i]]$Country
            medals <- cbind(data[[i]]$Gold, data[[i]]$Silver, data[[i]]$Bronze)
            currange <- (nK*(i-1)+1):(nK*i)
            res$winner[currange] <- sapply(allK, weightedMedalsK, medals=medals, unit=unit)
            res$year[currange] <- rep( (i-1)/(nK-1), nK)
            res$k[currange] <- allK
        }
        res$yup <- res$year + 0.5*precision
        res$ylw <- res$year - 0.5*precision 
        res$kup <- res$k + 0.5*precision 
        res$klw <- res$k - 0.5*precision 
        return(res)
        
    } else {
        
        ## a single year
        unit <- data[[1]]$Country
        medals <- cbind(data[[1]]$Gold, data[[1]]$Silver, data[[1]]$Bronze)
        wr <- seq(0, 1, by=precision)
        
        winners <- lapply(wr, function(x, medals=medals, precision=precision, unit=unit) {
            weights <- c(1, x); w <- seq(0, x, by=precision);
            sapply(w, weightedMedals, medals=medals, weights=weights, unit=unit)},
                          medals=medals, precision=precision, unit=unit)
        
        ## Convert winners to matrix format, add vertices
        levels <- seq(0, 1, by=precision)
        nlevels <- length(levels)
        ncells <- sum(nlevels:1)
        res <- as.data.frame(matrix(NA, nrow=ncells, ncol=4))
        names(res) <- c("winner","color","s","b")
        ctr <- 0
        for (i in 1:length(winners)) {
            for (j in 1:length(winners[[i]])) {
                ctr <- ctr+1
                res$winner[ctr] <- winners[[i]][j]
                res$s[ctr] <- levels[i]
                res$b[ctr] <- levels[j]
            }
        }
        
        res$sup <- res$s + precision/2
        res$slw <- res$s - precision/2
        res$bup <- res$b + precision/2
        res$blw <- res$b - precision/2
        
        return(res)
    }
}

## Unique winners
uniqueWinners <- function(data, precision=0.05) {
    winners <- getWinners(purgedata(), precision=0.125)$winner
    winners <- unique(unlist(strsplit(unique(data$winner),"\n")))
    winners
}

## Replace country names with 3 letter abbreviations
replaceCountries <- function(data, crosswalk) {
    n <- nrow(data)
    for (i in 1:n) {
        data$CountryFull <- data$Country
        data$Country[i] <- crosswalk$CTY[crosswalk$Country==data$Country[i]] 
    }
    data
}

## Restore full country name
fullcty <- function(x, crosswalk) {
    res <- sapply(as.list(x), function(x,crosswalk){
        crosswalk$Country[crosswalk$CTY==x]}, crosswalk=crosswalk)
    sapply(as.list(res), function(x) {
        if (x=="United States") {return("the United States")} else {
            if (x=="Netherlands") {return("the Netherlands")} else {
                if (x=="Soviet Union") {return("the Soviet Union")} else {
                    if (x=="Czech Republic") {return("the Czech Republic")} else {
                    return(x) }}}}
    })
}

## Restore short country name
shortcty <- function(x, crosswalk) {
    res <- sapply(as.list(x), function(x) {
        if (x=="the United States") {return("United States")} else {
            if (x=="the Netherlands") {return("Netherlands")} else {
                if (x=="the Soviet Union") {return("Soviet Union")} else {
                    if (x=="the Czech Republic") {return("Czech Republic")} else {
                                                                              return(x) }}}}
    })
    sapply(as.list(res), function(x,crosswalk){
        crosswalk$CTY[crosswalk$Country==x]}, crosswalk=crosswalk)
}


## Assign colors to countries
assignColors <- function(data, crosswalk, colors, pcolors, default=13) {
    uwin <- unique(data$winner)
    nwin <- length(uwin)
    uwincol <- rep(NA,nwin)
    for (i in 1:nwin) {
        col <- crosswalk$Color[uwin[i]==crosswalk$CTY]
        if (length(col)==1) {
            uwincol[i] <- col
        } else {
            uwincol[i] <- default
        }
    }

    for (i in 1:nrow(data)) {
        data$color[i] <- colors[uwincol[uwin==data$winner[i]]]
        data$pcolor[i] <- pcolors[uwincol[uwin==data$winner[i]]]
    }
    data
}

## Make a tile plot of weighted medal winners
plotWinners <- function(x, title=NULL, subtitle=NULL, exclusions=NULL, disqualified=NULL,
                        annotate=FALSE, verbose=TRUE, file="temp") {

    ## Construct plot limits from area around blocks of plot
    minx <- min(x$slw)
    maxx <- max(x$sup)
    limits <- c(minx,maxx,minx,maxx)
    
    ## Make data traces
    collectedPoints <- pointsTile(x=x$s, y=x$b, size=3.5, layer=20, pch=15, # 1.5
                                  col=x$pcolor, clip="off", plot=1)

    ##cex <- (nchar(x$winner)>=11)*0.275 + (nchar(x$winner)==7)*0.375 + (nchar(x$winner)==3)*0.45    
    cex <- (nchar(x$winner)>=11)*0.525 + (nchar(x$winner)==7)*0.625 + (nchar(x$winner)==3)*0.75    
    winnersTrace <- textTile(x=x$s, y=x$b, labels=x$winner, col=x$color, cex=cex, layer=1, plot=1)

    titleTrace <- textTile(x=0.25, y=0.95, labels=title, cex=1, plot=1)

    subtitleTrace <- textTile(x=0.25, y=0.90, labels=subtitle, cex=0.75, col="gray50", plot=1)

    ## List excluded countries in two smart text columns
    exclusionTrace <- pointsTile(x=NA,y=NA,plot=1)
    le <- length(exclusions)
    if (le) {
        if (le<=6) {
            exclX <- c(0.160, rep(0.080, le))
            exclY <- seq(from=0.500, by=-0.042, length.out=le+1)
            exclNum <- paste(1:le, exclusions, sep=". ")
        } else {
            if (le<10) {
                exclX <- c(0.160, rep(0.080, 6), rep(0.220, le-6))
                exclY <- c(seq(from=0.500, by=-0.042, length.out=7),
                           seq(from=0.500-0.042, by=-0.042, length.out=le-6))
                exclNum <- paste(1:le, exclusions, sep=". ")
            } else {
                exclX <- c(0.160, rep(0.080, 6), rep(0.220, 3))
                exclY <- c(seq(from=0.500, by=-0.042, length.out=7),
                           seq(from=0.500-0.042, by=-0.042, length.out=3))
                exclNum <- c(paste(1:7, exclusions[1:7], sep=". "),
                             "...",
                             paste(le, exclusions[le], sep=". "))
            }
        }
        exclusionTrace <- textTile(x=exclX, y=exclY,
                                   labels=c(paste("Placing", numbers2words(le+1, ordinal=TRUE), "behind"),
                                            exclNum),
                                   fontface=c("bold.italic", rep("plain",le)),
                                   cex=c(0.65, rep(0.55, le)),
                                   col="gray50", plot=1)
    }

    ## Note copyright
    copyrightTrace <- textTile(labels=paste0("\uA9 2022 Christopher Adolph\n", date()),
                               x=-0.135, y=-0.20, cex=0.4, col="gray50", clip="off", plot=1)
    
    ## Optional annotation traces
    geometricPoint <- geometricLine <- pointsTile(x=NA,y=NA,plot=1)
    verb1 <- verb2 <- verb3 <- verb4 <- verb5 <- verb6 <- verb7 <- verb8 <- pointsTile(x=NA,y=NA,plot=1)
    if (annotate) {
        geometricPoint <- pointsTile(x=0.5,y=0.25,size=3.4,col="gray50",layer=5,plot=1)
        geometricLine <- linesTile(x=seq(0,1,by=0.005),
                                   y=seq(0,1,by=0.005)^2,
                                   layer=5,
                                   col="gray50",
                                   plot=1
                                   )
        if (verbose) {
            verb1 <- textTile(labels="All medals equal\nG + S + B",
                              x=0.8, y=1.0, cex=0.6, plot=1)
            ## 0.8 0.985
            verb2 <- textTile(labels="Only\ngold\nG",
                              x=-0.02, y= 0.13, cex=0.6, clip="off", plot=1)
            ## -0.02  0.1
            verb3 <- textTile(labels="Geometric\nweights",
                              x=0.23, y=0.78, cex=0.6, plot=1)
            verb4 <- textTile(labels=expression(G + frac(S,k) + frac(B,k^2)),
                              x=0.37, y=0.78, cex=0.6, plot=1)
            verb5 <- textTile(labels="Goldilocks\nweights",
                              x=0.23, y=0.65, cex=0.6, plot=1)
            verb6 <- textTile(labels=expression(G + frac(S,2) + frac(B,4)),
                              x=0.37, y=0.65, cex=0.6, plot=1)
            verb7 <- linesTile(x=c(0.08,0.13), y=0.78, col="gray50",plot=1)
            verb8 <- pointsTile(x=0.105, y=0.65, size=1.5, col="gray50", plot=1)
        }
    }
    
    ## Make tile plot
    tile(collectedPoints,
         winnersTrace,
         titleTrace,
         subtitleTrace,
         exclusionTrace,
         copyrightTrace,
         geometricLine,
         geometricPoint,
         verb1, verb2, verb3, verb4, verb5, verb6, verb7, verb8,
         limits=limits,
         RxC=c(1,1),
         xaxis=list(major=FALSE),
         yaxis=list(label.loc=-0.5, major=FALSE),
         xaxistitle=list(labels="Value of Silver Relative to Gold", y=0.25),
         yaxistitle=list(labels="Value of Bronze Relative to Gold"),
         width=list(null=5, yaxistitle=4, yaxis.labelspace=-0.50),
         output=list(file=file, width=5.5, type="png")  ## 5.5
         )    
}

## Make a tile plot of geometric weighted medal winners for all years
plotWinnersAlltime <- function(x, title=NULL, subtitle=NULL, exclusions=NULL, availyears=NULL,
                               annotate=FALSE, verbose=TRUE, file="temp") {

    ## Construct plot limits from area around blocks of plot
    minx <- min(x$ylw)
    maxx <- max(x$yup)
    miny <- min(x$klw)
    maxy <- max(x$kup)
    limits <- c(minx,maxx,miny,maxy+0.05)

    ## List of year locations
    lyear <- unique(x$year)
    nyear <- length(lyear)
    
    ## Make data traces
    collectedPoints <- pointsTile(x=x$year, y=x$k, size=1.5, layer=20, pch=15,
                                  col=x$pcolor, clip="off", plot=1)
        
    cex <- (nchar(x$winner)==11)*0.275 + (nchar(x$winner)==7)*0.375 + (nchar(x$winner)==3)*0.45 
    winnersTrace <- textTile(x=x$year, y=x$k, labels=x$winner, col=x$color, cex=cex, layer=1, plot=1)

    goldilocksPoints <- pointsTile(x=NA,y=NA,plot=1)
    if (annotate) {
        goldilocksPoints <- pointsTile(x=lyear, y=rep(0.5, nyear),
                                       size=1.5, col="gray50", layer=5, clip="off", plot=1)
    }
        
    titleTrace <- textTile(x=0.15, y=1.1, labels=title, cex=1, clip="off", plot=1)

    subtitleTrace <- textTile(x=0.45, y=1.095, labels=subtitle, cex=0.75, col="gray50", clip="off", plot=1)

    ## List excluded countries in two smart text columns
    exclusionTrace <- pointsTile(x=NA,y=NA,plot=1)
    le <- length(exclusions)
    if (le) {
        if (le<=2) {
            exclX <- c(0.780, rep(0.680, le))
            exclY <- seq(from=1.15, by=-0.042, length.out=le+1)
            exclNum <- paste(1:le, exclusions, sep=". ")
        } else {
            if (le<=4) {
                exclX <- c(0.780, rep(0.680, 2), rep(0.790, le-2))
                exclY <- c(seq(from=1.15, by=-0.042, length.out=3),
                           seq(from=1.15-0.042, by=-0.042, length.out=le-2))
                exclNum <- paste(1:le, exclusions, sep=". ")
            } else {
                if (le<=6) {
                    exclX <- c(0.780, rep(0.680, 2), rep(0.800, 2), rep(0.920, le-4))
                    exclY <- c(seq(from=1.15, by=-0.042, length.out=3),
                               seq(from=1.15-0.042, by=-0.042, length.out=2),
                               seq(from=1.15-0.042, by=-0.042, length.out=le-4))
                    exclNum <- paste(1:le, exclusions, sep=". ")
                } else {
                    exclX <- c(0.780, rep(0.680, 2), rep(0.800, 2), rep(0.920, 2))
                    exclY <- c(seq(from=1.15, by=-0.042, length.out=3),
                               seq(from=1.15-0.042, by=-0.042, length.out=2),
                               seq(from=1.15-0.042, by=-0.042, length.out=2))
                    exclNum <- c(paste(1:5, exclusions[1:5], sep=". "),
                                 "...",
                                 paste(le, exclusions[le], sep=". "))                    
                }
            }
        }
        exclusionTrace <- textTile(x=exclX, y=exclY,
                                   labels=c("Excluded countries", exclNum),
                                   fontface=c("bold.italic", rep("plain",le)),
                                   cex=c(0.65, rep(0.55, le)),
                                   col="gray50", clip="off", plot=1)
    }

    ## Note copyright
    copyrightTrace <- textTile(labels=paste0("\nCopyright 2022,\nChristopher Adolph"),
                               x=-0.15, y=-0.15, cex=0.5, col="gray50", clip="off", plot=1)
    
    ## Year labels
    xseq <- seq(2, nyear, by=2)
    xat <- (xseq - 1)/(nyear - 1)
    xlabels <- paste0("'", substr(availyears[xseq],3,4))
    
    ## Make tile plot
    tile(collectedPoints,
         winnersTrace,
         goldilocksPoints,
         titleTrace,
         subtitleTrace,
         exclusionTrace,
         copyrightTrace,
         limits=limits,
         RxC=c(1,1),
         xaxis=list(major=FALSE, at=xat, labels=xlabels),
         yaxis=list(label.loc=-0.5, major=FALSE, at=seq(0, 1, by=0.1)), 
         xaxistitle=list(labels="Year", y=0.25),
         yaxistitle=list(labels="Value of Lower Medal Relative to Higher Medal"),
         width=list(null=5, yaxistitle=4, yaxis.labelspace=-0.50),
         output=list(file=file, width=5.5, type="png")
         )   
}


## Make Medal Table for output
makeMedalsTable <- function(data, disqualified, olympics, crosswalk, raw) {

    raw <- raw[[1]]
    
    ## Remove disqualified countries
    if (length(disqualified))
        for (i in 1:length(disqualified))
            raw <- raw[raw$Country!=disqualified[i], ]

    ## Replace country names
    for (i in 1:nrow(raw)) {
        raw$Country[i] <- firstCap(fullcty(raw$Country[i], crosswalk))
        if (substr(raw$Country[i], 1, 3)=="The")
            raw$Country[i] <- substr(raw$Country[i], 5, nchar(raw$Country[i]))
    }
    
    ## Make goldilocks
    goldilocks <- 4*raw$Gold + 2*raw$Silver + 1*raw$Bronze
    
    ## Make total
    total <- raw$Gold + raw$Silver + raw$Bronze
    
    ## Make table
    res <- as.data.frame(cbind(goldilocks, raw$Gold, raw$Silver, raw$Bronze, total))
    res <- cbind(raw$Country, res)
    names(res) <- c("Country", "Goldilocks", "Gold", "Silver", "Bronze", "Total")
    
    ## Sort data frame by default:  first by goldilocks, then by gold, then total
    res <- res[with(res, rev(order(res[,2], res[,3], res[,5]))), ]

    ## Create ranks
    Rank <- rep(NA, nrow(res))
    Rank[1] <- 1
    for (i in 2:nrow(res)) {
        if (identical(res$Goldilocks[i], res$Goldilocks[i-1])) {
            Rank[i] <- Rank[i-1]
        } else {
            Rank[i] <- i
        }
    }

    ## Add ranks to table
    res <- cbind(Rank, res)
    
    res
}


## Make captions
makeCaption <- function(data, exclusions, disqualified, year, olympics, crosswalk, raw) {

    ## Helper to lookup a given country's medals
    getMedalTable <- function(str, raw) {
        res <- NULL
        for (i in 1:length(str))
            res <- rbind(res, raw[raw$Country==str[i],])
        res <- cbind(res$Gold, res$Silver, res$Bronze)
        res
    }
    
    ## Check to see if single or all years
    if (length(year)>1) { # multi year caption
        res <- "The circled countries are each year's goldilocks winners.  The gold medal leaders are listed in the bottom row of boxes, and the total medal leaders in the top row."
    } else { # single year caption       

        ## Clean up raw
        raw <- raw[[1]]
        
        ## Check for disqualified
        ndq <- length(disqualified)+1
        if (ndq-1) {
            ##if (length(disqualified)>1) verbtense <- "are" else verbtense <- "is"
            disqStr <- paste("With", conjPaste(fullcty(disqualified, crosswalk)), "disqualified,")
            dq <- TRUE
        } else {
            disqStr <- ""
            dq <- FALSE
        }    
        
        ## Check for exclusions
        placement <- length(exclusions)+1
        if (placement-1) {

            ## Add medal totals to string(s)
            curexclMedals <- getMedalTable(exclusions, raw)
            for (i in 1:length(exclusions))
                exclusions[i] <- paste0(fullcty(exclusions[i], crosswalk),
                                        " (", paste0(curexclMedals[i,], collapse="/"), ")")
            exclusions <- conjPaste(exclusions)
            
            exclStr <- paste("this assumes", exclusions,
                             "placed first")
            if (placement>2)
                exclStr <- paste(exclStr, ifelse(placement==3, "and", "through"),
                                 numbers2words(placement-1, ordinal=TRUE))
            exclStr <- paste0(exclStr, ", which may be debatable.")
        } else {
            exclStr <- ""
        }

        ## Check for multiple winners
        setwin <- unique((data$winner))
        nsetwin <- length(setwin)
        ctywin <- ctywin0 <- fullcty(unique(unlist(strsplit(unique(data$winner),"\n"))), crosswalk)
        nctywin <- length(ctywin)

        ctywinStr <- conjPaste(ctywin)

        ctywinMedals <- getMedalTable(unique(unlist(strsplit(unique(data$winner),"\n"))), raw)
        for (i in 1:nctywin)
            ctywin[i] <- paste0(ctywin[i], " (", paste0(ctywinMedals[i,], collapse="/"), ")")
        ctywinStr <- conjPaste(ctywin)
        
        ## One country always best
        if (nctywin==1) {
            if (disqStr=="") ctywinStr <- firstCap(ctywinStr)
            res <- paste("<strong><em>No contest!</em></strong></br><p style='line-height:0.4'></br></p>",
                         disqStr,
                         ctywinStr, "was the clear",
                         numbers2words(placement, ordinal=TRUE),
                         "place medal leader in the", year, simpleCaps(olympics),
                         "Olympics, regardless of the relative value of gold, silver, and bronze medals.",
                         "</br><p style='line-height:0.4'></br></p>",
                         firstCap(exclStr))
        } else {  # Multiple winners

            ## The same countries always best (tie)
            if (nsetwin==1) {
                if (disqStr=="") ctywinStr <- firstCap(ctywinStr)
                res <- paste("<strong><em>It's a tie!</em></strong></br><p style='line-height:0.4'></br></p>",
                             disqStr,
                             ctywinStr, "tied as the",
                             numbers2words(placement, ordinal=TRUE),
                             "place medal leaders in the", year, simpleCaps(olympics),
                             "Olympics, regardless of the relative value of gold, silver, and bronze medals.",
                             "</br><p style='line-height:0.4'></br></p>",
                             firstCap(exclStr))
            } else {  # Winner(s) depend on weights
                
                wgtcaptions <- function(data, s, b, descS, descP=NULL, showTotal="none", raw=NULL) {
                    if (!is.na(s)&&!is.na(b)) {
                        curwin <- data$winner[(data$s==s)&(data$b==b)]
                        curwin <- curwin0 <- fullcty(unlist(strsplit(curwin, "\n")), crosswalk)
                        
                        ## Add medal totals to string(s)
                        curwinMedals <- getMedalTable(shortcty(curwin, crosswalk), raw)
                        for (i in 1:length(curwin))
                            curwin[i] <- paste0(curwin[i], " (", paste0(curwinMedals[i,], collapse="/"), ")")
                        curwin <- conjPaste(curwin)
                        
                        if (length(curwin0)>1)
                            res <- paste(conjPaste(curwin), descP)
                        else
                            res <- paste(curwin, descS)                        
                    } else {
                        cases <- fullcty(unique(unlist(strsplit(paste(data$winner[(data$s==0)&(data$b==0)],
                                                  data$winner[(data$s==0.5)&(data$b==0.25)],
                                                  data$winner[(data$s==1)&(data$b==1)], sep="\n"), "\n"))),
                                         crosswalk)
                        curwin <- setdiff(ctywin0, cases)

                        if (length(curwin)) {
                            ## Add medal totals to string(s)
                            curwinMedals <- getMedalTable(shortcty(curwin, crosswalk), raw)
                            for (i in 1:length(curwin))
                                curwin[i] <- paste0(curwin[i], " (", paste0(curwinMedals[i,], collapse="/"), ")")
                            curwin <- conjPaste(curwin)
                            res <- paste(conjPaste(curwin, "or"), descS)
                        } else {
                            res <- ""
                        }
                    }
                    res
                    #firstCap(res)
                }

                if (exclStr!="") exclStr <- paste("All", exclStr)

                ## Check for same winners under three criteria
                w1 <- trimws(wgtcaptions(data, 0.5, 0.25, "", "", raw=raw))
                w2 <- trimws(wgtcaptions(data, 0, 0, "", "", raw=raw))
                w3 <- trimws(wgtcaptions(data, 1, 1, "", "", raw=raw))
                if ((w1==w2)&&(w2==w3)) clearcut <- TRUE else clearcut <- FALSE
                if (clearcut) howmany <- length(unlist(strsplit(w1, "\n")))
                
                goldilocks <- wgtcaptions(data, 0.5, 0.25,
                                          "was the goldilocks leader,",
                                          "were the goldilocks leaders,", raw=raw)
                mostgold <- paste0(wgtcaptions(data, 0, 0,
                                          "was the gold medal leader",
                                          "were the gold medal leaders", raw=raw), ", and")
                mostmedals <- paste0(wgtcaptions(data, 1, 1,
                                          "was the overall medal leader",
                                          "were the overall medal leaders", raw=raw), ".</br><p style='line-height:0.4'></br></p>")
                othwinners <- firstCap(wgtcaptions(data, NA, NA,
                                          paste("could also be considered the",
                                                numbers2words(placement, ordinal=TRUE),
                                                "place finisher under other valid weightings.</br><p style='line-height:0.4'></br></p>"),
                                          raw=raw))
                
                if (dq||clearcut) {
                    nwin <- numbers2words(length(ctywin))
                } else {
                    nwin <- firstCap(numbers2words(length(ctywin)))
                }

                if (clearcut) {
                    if (howmany==1) {
                        sumtext <- "<strong><em>There&rsquo;s a clear winner.</em></strong></br><p style='line-height:0.4'></br></p>"
                        verbtense <- "is"
                        singplur <- "clear winner:"
                    } else {
                        sumtext <- paste("<strong><em>There are", numbers2words(howmany), "clear winners.</em></strong></br><p style='line-height:0.4'></br></p>")
                        verbtense <- "are"
                        singplur <- "clear winners:"
                    }
                    res <- paste(sumtext,
                                 disqStr,
                                 "Although",
                                 nwin,
                                 "different countries can claim",
                                 numbers2words(placement, ordinal=TRUE),
                                 "place in the", year, simpleCaps(olympics),
                                 "Olympics depending on the relative value of gold, silver, and bronze medals,",
                                 "arguably there", verbtense, numbers2words(howmany), singplur,
                                 paste(firstCap(conjPaste(unlist(strsplit(w1, "\n")))), ".", sep=""),
                                 "</br><p style='line-height:0.4'></br></p>",
                                 firstCap(goldilocks), mostgold, mostmedals, othwinners, exclStr)
                } else {
                    sumtext <- "<strong><em>Controversy!</em></strong></br><p style='line-height:0.4'></br></p>"
                    res <- paste(sumtext,
                             disqStr,
                             nwin,
                             "different countries can claim",
                             numbers2words(placement, ordinal=TRUE),
                             "place in the", year, simpleCaps(olympics),
                             "Olympics depending on the relative value of gold, silver, and bronze medals.</br><p style='line-height:0.4'></br></p>",
                             firstCap(goldilocks), mostgold, mostmedals, othwinners, exclStr)
                }
                                    
            }
        }
    }

    ## Capitalize first letter
    res <- paste0(toupper(substr(res, 1, 1)), substr(res, 2, nchar(res)))
    res
}

## Load explanatory text in html format
makeExplanation <- function() {
    read_file("explanation.txt")
}

## Capitalize first letter of each word in string
simpleCaps <- function(x) {
  s <- strsplit(x, " ")[[1]]
  paste(toupper(substring(s, 1,1)), substring(s, 2),
      sep="", collapse=" ")
}

## Capitalize first word only
firstCap <- function(x){
    paste0(toupper(substring(x,1,1)), substring(x,2))
}
    
## Convert c("A", "B", "C") to "A, B, and C", etc.
conjPaste <- function(x, conj="and"){
    lx <- length(x)
    if (lx==1) {
        x
    } else {
        if (lx==2) {
            paste(x, collapse=paste0(" ", conj, " "))
        } else {
            paste0(paste(x[1:(lx-1)], collapse=", "), ", ", conj, " ", x[lx])
        }   
    }
}


