# Weighted Olympic Medal Counts
# Interactive Visualization
# R Shiny Server
#
# Christopher Adolph
# 27 February 2023   faculty.washington.edu/cadolph

# Load libraries and helper functions
library(shinyjs)
library(RColorBrewer)
##devtools::install_github("chrisadolph/tileForShiny")
library(tile)
library(DT)
library(shinycssloaders)
library(readr)
source("helperShiny.R")
source("numbers2words.R") # needs work for ordinal cases >99

## Nice colors
col <- c(brewer.pal(9,"Set1")[c(1:5,7:8)], brewer.pal(8,"Set2")[1:5], "#000000")
pcol <- c(brewer.pal(9,"Pastel1")[c(1:5,7:8)], brewer.pal(8,"Pastel2")[1:5], lighten("#000000"))

## Load country abbreviations
crosswalk <- read.csv("data/countries.csv", header=TRUE, stringsAsFactors=FALSE)

## Load data and replace country names with abbr: Winter
availyearsW <- c(1924, 1928, 1932, 1936, 1948,
                1952, 1956, 1960, 1964, 1968,
                1972, 1976, 1980, 1984, 1988,
                1992, 1994, 1998, 2002, 2006,
                2010, 2014, 2018, 2022)
for (i in 1:length(availyearsW)) {
    file <- paste0("winter", availyearsW[i])
    assign(file, read.csv(paste("data/", file, ".csv", sep=""), header=TRUE, stringsAsFactors=FALSE))
    assign(file, replaceCountries(get(file),crosswalk))
}

##availyearsW <- c(availyearsW, 2022)
##winter2022 <- scrape2022()
##winter2022 <- replaceCountries(winter2022, crosswalk)

## Load data and replace country names with abbr: Summer
availyearsS <- c(2000, 2004, 2008, 2012)
for (i in 1:length(availyearsS)) {
    file <- paste0("summer", availyearsS[i])
    assign(file, read.csv(paste("data/", file, ".csv", sep=""), header=TRUE, stringsAsFactors=FALSE))
    assign(file, replaceCountries(get(file),crosswalk))
}

## Run shiny server
shinyServer(function(input, output, session) {

    ## Check for year (depends on Olympics)
    checkyear <- reactive({
        if (input$olympics=="winter") {
            res <- olympiadW(input$yearW)
            ##if (input$period=="year") {res <- olympiadW(input$yearW)} else {res <- availyearsW}
        } else {
            res <- olympiadS(input$yearS)
            ##if (input$period=="year") {res <- olympiadS(input$yearS)} else {res <- availyearsS}
        }
        return(res)
    })
        
    ## Grab relevant data for Olympics & years
    getdata <- reactive({
        y <- checkyear()

        ##if (y==2018) {
            ##winter2018 <- scrape2018()
            ##winter2018 <- replaceCountries(winter2018, crosswalk)
        ##}
        
        ly <- length(y)
        dy <- vector("list", ly)
        for (i in 1:ly) {
            dy[[i]] <- get(paste(input$olympics, y[i], sep=""))
        }

        ## Sneaky save the date stamp
        ##if (nrow(dy[[1]])>1) row.names(dy[[1]]) <- c(date(), 2:nrow(dy[[1]])) else row.names(dy[[1]]) <- date()
        
        dy
    })

    ## Purge unused countries based on UI
    purgedata <- reactive({

        ##print(getdata())
        
        if (input$disqualifyRUS&&(input$yearW==22)) currdisqualified <- "RUS" else currdisqualified <- NULL
        excludeCountry(getdata(), c(input$excluded, currdisqualified))
        ##excludeCountry(getdata(), c(input$excluded, input$disqualified))
        })
    
    ## Compute winners and colors
    processdata <- reactive(       
        assignColors(getWinners(purgedata(),
                                precision=0.125),
                     crosswalk,
                     col,
                     pcol)
    )
    
    ## Create country exclusion UI panel
    excl <- reactive({
        currexcluded <- c(input$excluded)
        excludable0 <- unique(processdata()$winner)
        excludable0 <- sort(unique(unlist(strsplit(excludable0,"\n"))))
        excludable0 <- unique(c(currexcluded, excludable0))
        excludable <- as.list(excludable0)
        names(excludable) <- excludable0
        list(excludable=excludable, currexcluded=currexcluded)
        })


    ##isolate(unlist(excl()$excludable)),
    ##isolate(uniqueWinners(purgedata(), precision=0.125)),
    observe({
    updateSelectizeInput(session, "excluded",
                         choices = crosswalk$CTY,
                         selected = isolate(excl()$currexcluded),
                         server = TRUE)
    })

    disq <- reactive({
        ##currdisqualified <- c(input$disqualified)
        if (input$disqualifyRUS&&(input$yearW==22)) currdisqualified <- "RUS" else currdisqualified <- NULL
        disqualifiable0 <- unique(processdata()$winner)
        disqualifiable0 <- sort(unique(unlist(strsplit(disqualifiable0,"\n"))))
        disqualifiable0 <- unique(c(currdisqualified, disqualifiable0))
        disqualifiable <- as.list(disqualifiable0)
        names(disqualifiable) <- disqualifiable0        
        list(disqualifiable=disqualifiable, currdisqualified=currdisqualified)
        })
    
    updateSelectizeInput(session, "disqualified",
                         choices = crosswalk$CTY,
                         selected = isolate(disq()$currdisqualified),
                         server = TRUE)   
        
    ## Make a dynamically-sized plot as a png file
    output$myImage <- renderImage({
        
        ## Get title
        title <- paste(simpleCaps(input$olympics), "Olympics")
        subtitle <- "ALL EVENTS" # to generalize later
   
        ## A temp file to save the output
        outfile <- tempfile()

        ## Make plot -- year by year
        ##if (input$period=="year") {
            plotWinners(processdata(),
                        title=paste(checkyear(), title),
                        subtitle=subtitle,
                        exclusions=excl()$currexclude,
                        disqualified=disq()$currdisqualified,
                        annotate=input$annotate,
                        file=outfile)
        ##}

        ## Make plot -- all time, all k
        ##if (input$period=="all") {
        ##    plotWinnersAlltime(processdata(),
        ##                       title=title,
        ##                       subtitle=subtitle,
        ##                       exclusions=excl()$currexclude,
        ##                       annotate=input$annotate,
        ##                       availyears=checkyear(),
        ##                       file=outfile)
        ##}
        
        ## Return a list containing the filename
        list(src = paste0(outfile, ".png"),
             width = 1.045627*700,  ## 825
             height = 700, ## 789
             alt = "(c) 2022 Christopher Adolph")
    }, deleteFile = TRUE)

    output$caption <- renderUI(HTML(makeCaption(processdata(),
                                             excl()$currexclude,
                                             disq()$currdisqualified,
                                             checkyear(),
                                             input$olympics,
                                             crosswalk,
                                             getdata()
                                             ))
                               )

    output$medalTable <- DT::renderDataTable({
        makeMedalsTable(processdata(),
                        disq()$currdisqualified,
                        input$olympics,
                        crosswalk,
                        getdata()
                        )
    }, options = list(orderClasses = TRUE,
                      bLengthChange=0,  ## show/hide records per page dropdown
                      bFilter=0,  ## global search box on/off
                      aoColumns = list(
                          list(NULL),
                          list(NULL),
                          list(orderSequence = c('desc')),
                          list(orderSequence = c('desc')),
                          list(orderSequence = c('desc')),
                          list(orderSequence = c('desc')),
                          list(orderSequence = c('desc'))
                      )),
                      ##columnDefs = list(list(orderSequence = c('desc'), targets = "_all"))),
    rownames= FALSE)

    output$tableTitle <- renderUI(HTML(paste("<h1>A Better, Goldilocks-Based Medals Table for the",
                                       checkyear(),
                                       "Winter Olympics</h1>")))

    output$explanation <- renderUI(withMathJax(HTML(makeExplanation()))
                               )
})
